#include <GMLuaModule.h>
#include <igameevents.h>

#define LS lua_State *L
#define GEVENTS_NAME "IGameEvent"
#define GEVENTS_TYPE 9085

GMOD_MODULE(Load, Unload);

ILuaInterface *gLua = NULL;
IGameEventManager2 *gameeventmanager = NULL;

class CGameEventListener : public IGameEventListener2
{
	public:
		~CGameEventListener() { gameeventmanager->RemoveListener(this); };
		
		void FireGameEvent( IGameEvent *event )
		{
			gLua->Push(gLua->GetGlobal("hook")->GetMember("Call"));
			gLua->Push(event->GetName());
			gLua->PushNil();
			gLua->PushUserData(gLua->GetMetaTable(GEVENTS_NAME, GEVENTS_TYPE), event);
			
			gLua->Call(3, 0);
		};
};

CGameEventListener CEventListener;

LUA_FUNCTION(Create)
{
	gLua->CheckType(1, GLua::TYPE_STRING);

	IGameEvent *event = gameeventmanager->CreateEvent(gLua->GetString(1));

	if (event)
		gLua->PushUserData(gLua->GetMetaTable(GEVENTS_NAME, GEVENTS_TYPE), event);
	else
		gLua->PushNil();

	return 1;
}

LUA_FUNCTION(Name)
{
	gLua->CheckType(1, GEVENTS_TYPE);

	IGameEvent *event = (IGameEvent*)gLua->GetUserData(1);

	if (event)
		gLua->Push(event->GetName());
	else
		gLua->PushNil();

	return 1;
}

LUA_FUNCTION(Fire)
{
	gLua->CheckType(1, GEVENTS_TYPE);

	IGameEvent *event = (IGameEvent*)gLua->GetUserData(1);

	if (event)
		gameeventmanager->FireEvent(event);

	return 0;
}

LUA_FUNCTION(FireClientSide)
{
	gLua->CheckType(1, GEVENTS_TYPE);

	IGameEvent *event = (IGameEvent*)gLua->GetUserData(1);

	if (event)
		gameeventmanager->FireEventClientSide(event);
		
	return 0;
}

LUA_FUNCTION(GetBool)
{
	gLua->CheckType(1, GEVENTS_TYPE);
	gLua->CheckType(2, GLua::TYPE_STRING);
	
	IGameEvent *event = (IGameEvent*)gLua->GetUserData(1);

	if (event)
		gLua->Push(event->GetBool(gLua->GetString(2)));
	else
		gLua->PushNil();

	return 1;
}

LUA_FUNCTION(SetBool)
{
	gLua->CheckType(1, GEVENTS_TYPE);
	gLua->CheckType(2, GLua::TYPE_STRING);
	gLua->CheckType(3, GLua::TYPE_BOOL);

	IGameEvent *event = (IGameEvent*)gLua->GetUserData(1);

	if (event)
		event->SetBool(gLua->GetString(2), gLua->GetBool(3));

	return 0;
}

LUA_FUNCTION(GetString)
{
	gLua->CheckType(1, GEVENTS_TYPE);
	gLua->CheckType(2, GLua::TYPE_STRING);
	
	IGameEvent *event = (IGameEvent*)gLua->GetUserData(1);

	if (event)
		gLua->Push(event->GetString(gLua->GetString(2)));
	else
		gLua->PushNil();

	return 1;
}

LUA_FUNCTION(SetString)
{
	gLua->CheckType(1, GEVENTS_TYPE);
	gLua->CheckType(2, GLua::TYPE_STRING);
	gLua->CheckType(3, GLua::TYPE_STRING);

	IGameEvent *event = (IGameEvent*)gLua->GetUserData(1);

	if (event)
		event->SetString(gLua->GetString(2), gLua->GetString(3));

	return 0;
}

LUA_FUNCTION(GetInt)
{
	gLua->CheckType(1, GEVENTS_TYPE);
	gLua->CheckType(2, GLua::TYPE_STRING);
	
	IGameEvent *event = (IGameEvent*)gLua->GetUserData(1);

	if (event)
		gLua->PushDouble(event->GetInt(gLua->GetString(2)));
	else
		gLua->PushNil();

	return 1;
}

LUA_FUNCTION(SetInt)
{
	gLua->CheckType(1, GEVENTS_TYPE);
	gLua->CheckType(2, GLua::TYPE_STRING);
	gLua->CheckType(3, GLua::TYPE_NUMBER);

	IGameEvent *event = (IGameEvent*)gLua->GetUserData(1);

	if (event)
		event->SetInt(gLua->GetString(2), gLua->GetInteger(3));

	return 0;
}

LUA_FUNCTION(GetFloat)
{
	gLua->CheckType(1, GEVENTS_TYPE);
	gLua->CheckType(2, GLua::TYPE_STRING);
	
	IGameEvent *event = (IGameEvent*)gLua->GetUserData(1);

	if (event)
		gLua->Push(event->GetFloat(gLua->GetString(2)));
	else
		gLua->PushNil();
	
	return 1;
}

LUA_FUNCTION(SetFloat)
{
	gLua->CheckType(1, GEVENTS_TYPE);
	gLua->CheckType(2, GLua::TYPE_STRING);
	gLua->CheckType(3, GLua::TYPE_NUMBER);

	IGameEvent *event = (IGameEvent*)gLua->GetUserData(1);

	if (event)
		event->SetFloat(gLua->GetString(2), gLua->GetNumber(3));

	return 0;
}

LUA_FUNCTION(InitHook)
{
	gLua->CheckType(1, GLua::TYPE_STRING);

	if (! gameeventmanager->FindListener(&CEventListener, gLua->GetString(1)))
		gameeventmanager->AddListener(&CEventListener, gLua->GetString(1), gLua->IsServer());
	
	return 0;
}

int Load(LS)
{
	gLua = Lua();

	CreateInterfaceFn engineI = Sys_GetFactory("engine.dll");
	gameeventmanager = (IGameEventManager2*)engineI(INTERFACEVERSION_GAMEEVENTSMANAGER2, NULL);
	
	ILuaObject *mevents = gLua->GetMetaTable(GEVENTS_NAME, GEVENTS_TYPE);
		ILuaObject *__index = gLua->GetNewTable();
			__index->SetMember("Name", Name);
			__index->SetMember("Fire", Fire);
			__index->SetMember("FireClientSide", FireClientSide);

			__index->SetMember("GetBool", GetBool);
			__index->SetMember("SetBool", SetBool);
			__index->SetMember("GetString", GetString);
			__index->SetMember("SetString", SetString);
			__index->SetMember("GetInt", GetInt);
			__index->SetMember("SetInt", SetInt);
			__index->SetMember("GetFloat", GetFloat);
			__index->SetMember("SetFloat", SetFloat);

			mevents->SetMember("__index", __index);
		__index->UnReference();
	mevents->UnReference();

	gLua->NewGlobalTable("gevent");

	ILuaObject *gameeventsT = gLua->GetGlobal("gevent");
		gameeventsT->SetMember("Create", Create);
		gameeventsT->SetMember("InitHook", InitHook);
	gameeventsT->UnReference();

	return 0;
}

int Unload(LS)
{
	return 0;
}